const dataController = require('./dataController');
const fileController = require('./fileController');
const usersController = require('./usersController');
const postParser = require('./postParser');

const routes = [
    {
        method: 'POST',
        url: '/api/photos/filters',
        loginRequired: true,
        fun: async (req) => {
            let obj = await postParser.parseBody(req, true);
            if (!obj || Object.keys(obj).length == 0)
                return [400, { message: 'unable to parse JSON' }];
            if (obj.photoid == undefined)
                return [400, { message: 'photo id not specified' }];

            let photo = dataController.getPhoto(obj.photoid);
            if (!photo || !photo.file || !photo.file.path) 
                return [404, { message: 'photo not found' }];

            let path = obj.customSource ?? photo.file.path;
            let newpath = await fileController.applyFilter(path, obj.filter, obj.params);

            switch(newpath) {
                case -1: return [400, { message: 'invalid filter name' }];
                case -2: return [400, { message: 'invalid params count' }];
                case -3: return [404, { message: 'photo not found (2)' }];
                case -4: return [400, { message: 'invalid param type' }];
                case -5: return [400, { message: 'invalid param value' }];
                case -6: return [500, { message: 'error while applying filter' }];
            }

            dataController.addFilterHistory(obj.photoid, obj.filter, newpath)
            
            return [200, { message: 'success' }];
        }
    },
    {
        method: 'GET',
        url: '/api/photos/filters',
        fun: async (req) => {

            let obj = fileController.getAvailableFilters();
            return [200, obj];
        }
    },
    {
        method: 'POST',
        loginRequired: true,
        url: '/api/photos',
        fun: async (req) => {
            let saveStatus = await fileController.sendPhoto(req);
            if (saveStatus.err || !saveStatus.file)
                return [500, { message: 'error while uploading' }];

            let userID = usersController.getCurrentUserID(req);
            let photo = dataController.addPhoto(saveStatus.file, userID);
            return [200, photo];
        },
    },
    {
        method: 'GET',
        loginRequired: true,
        url: '/api/photos',
        fun: async (req) => {
            let userID = usersController.getCurrentUserID(req);
            let photos = dataController.getPhotos(userID);
            return [200, photos];
        },
    },
    {
        method: 'GET',
        url: '/api/photos/tags/',
        paramAfterURL: true,
        fun: async (req, param) => {
            let tags = dataController.getPhotoTags(param);
            return tags ? [200, tags] : [404, { message: 'photo not found' }];
        }
    },
    {
        method: 'GET',
        url: '/api/photos/meta/',
        paramAfterURL: true,
        fun: async (req, param) => {
            let photo = dataController.getPhoto(param);
            if (!photo) return [404, { message: 'photo not found' }];
            let meta = await fileController.getMeta(photo.file.path);
            return meta ? [200, meta] : [500, { message: 'could not get metadata' }];
        }
    },
    {
        method: 'GET',
        url: '/api/photos/',
        paramAfterURL: true,
        fun: async (req, param) => {
            let photo = dataController.getPhoto(param);
            return photo ? [200, photo] : [404, { message: 'photo not found' }];
        }
    },
    {
        method: 'PATCH',
        url: '/api/photos',
        loginRequired: true,
        fun: async (req) => {
            let obj = await postParser.parseBody(req, true);
            if (!obj || Object.keys(obj).length == 0)
                return [400, { message: 'unable to parse JSON' }];
            let success = dataController.modifyPhotoJSON(obj.id, obj);
            return success ? [200, { message: 'success' }] : [404, { message: 'photo not found' }];
        }
    },
    {
        method: 'PATCH',
        url: '/api/photos/tags',
        loginRequired: true,
        fun: async (req) => {
            let obj = await postParser.parseBody(req, true);
            if (!obj || Object.keys(obj).length == 0)
                return [400, { message: 'unable to parse JSON' }];
            if (!obj.tag || obj.tag.length < 1)
                return [400, { message: 'no tag provided' }];

            let success = dataController.addTagsToPhoto(obj.photoid, [obj.tag]);
            return success ? [200, { message: 'success' }] : [404, { message: 'photo not found' }];
        }
    },
    {
        method: 'PATCH',
        url: '/api/photos/tags/mass',
        loginRequired: true,
        fun: async (req) => {
            let obj = await postParser.parseBody(req, true);
            if (!obj || Object.keys(obj).length == 0)
                return [400, { message: 'unable to parse JSON' }];
            if (obj.tags.length < 1)
                return [400, { message: 'no tags provided' }];

            let success = dataController.addTagsToPhoto(obj.photoid, obj.tags);
            return success ? [200, { message: 'success' }] : [404, { message: 'photo not found' }];
        }
    },
    {
        method: 'DELETE',
        url: '/api/photos/tags',
        loginRequired: true,
        fun: async (req) => {
            let obj = await postParser.parseBody(req, true);
            if (!obj || Object.keys(obj).length == 0)
                return [400, { message: 'unable to parse JSON' }];
            if (!obj.tag || obj.tag.length < 1)
                return [400, { message: 'no tag provided' }];

            let success = dataController.removeTagsFromPhoto(obj.photoid, [obj.tag]);
            return success ? [200, { message: 'success' }] : [404, { message: 'photo not found' }];
        }
    },
    {
        method: 'DELETE',
        url: '/api/photos/',
        loginRequired: true,
        paramAfterURL: true,
        fun: async (req, param) => {
            let filepath = dataController.delPhoto(param);
            if (filepath) fileController.deletePhoto(filepath);
            return filepath ? [200, { message: 'success' }] : [404, { message: 'photo not found' }];
        }
    },
            
]

module.exports = routes;