const bcrypt = require('bcryptjs');
const jwt = require('jsonwebtoken');
const tracer = require('tracer').colorConsole();
require('dotenv').config();

const users = {};
const userEmails = {};
const expiredTokens = [];
let currentUserID = 0;

async function hashPassword(password) {
    return await bcrypt.hash(password, 10);
}

async function registerUser(name, lastname, email, password) {
    if (userEmails[email] !== undefined) return false;

    const obj = {
        name, lastname, email,
        password: await hashPassword(password),
        confirmed: false,
        id: currentUserID
    }
    users[currentUserID] = obj;
    userEmails[email] = currentUserID;
    currentUserID++;
    return obj;
}

function confirmUser(token) {
    let decoded = readToken(token);
    if (!decoded) return false;
    if (!decoded.type == "confirm") return;
    let userID = userEmails[decoded.data];
    if (userID == undefined) return false;

    users[userID].confirmed = true;
    return true;
}

function createToken(data, type) {

    let token = jwt.sign(
        {
            data,
            type
        },
        process.env.PRIVATE_KEY,
        {
            expiresIn: "1d"
        }
    );
    return token;
}

function readToken(token) {
    try {
        let decoded = jwt.verify(token, process.env.PRIVATE_KEY)
        return decoded;
    }
    catch (ex) {
        console.warn(ex);
        return null;
    }
}

async function loginUser(email, password) {
    let user = users[userEmails[email]];
    if (!user) return 0;
    let passcorrect = await bcrypt.compare(password, user.password);
    if (!passcorrect) return 1;
    if (!user.confirmed) return 2;

    let sessionToken = createToken(user.id, 'login');
    return sessionToken;
}

function checkLogin(req) {
    if (!req.headers.authorization) return false;
    if (!req.headers.authorization.startsWith("Bearer")) return false;

    let token = req.headers.authorization.split(" ")[1];
    if (expiredTokens.includes(token)) return false;
    let decoded = readToken(token);
    if (!decoded || !decoded.type == 'login') return false;
    if (!users[decoded.data]) return false;

    return true;
    //return user.email;
}

function getCurrentUserID(req) {
    let token = req.headers.authorization.split(" ")[1]
    let decoded = readToken(token);
    if (!decoded || !decoded.type == 'login') return -1;
    if (!users[decoded.data]) return -1;

    return decoded.data;
}

function getUser(id) {
    return users[id];
}

function debugGetUsers() {
    return users;
}

function addAvatar(id, file) {
    let user = users[id];
    if (!user) return false;

    user.avatar = file;

    return true;
}

function logout(req) {
    if (!req.headers.authorization) return false;
    if (!req.headers.authorization.startsWith("Bearer")) return false;

    let token = req.headers.authorization.split(" ")[1]
    expiredTokens.push(token);

    return true;
}

async function updateUser(id, obj) {
    let user = users[id];
    if (!user) return 1;

    if (obj.password) {
        if (!obj.oldPass) return 3;
        let passcorrect = await bcrypt.compare(obj.oldPass, user.password);
        if (!passcorrect) return 3;
        user.password = await hashPassword(obj.password);
    }
    if (obj.name) user.name = obj.name;
    if (obj.lastname) user.lastname = obj.lastname;
    if (obj.email) {
        if (user.email == obj.email) return 0;
        if (userEmails[obj.email] !== undefined) return 2;
        userEmails[user.email] = undefined;
        userEmails[obj.email] = id;
        user.email = obj.email;
    }

    return 0;
}

module.exports = {
    registerUser,
    createToken,
    readToken,
    confirmUser,
    loginUser,
    checkLogin,
    getCurrentUserID,
    getUser,
    addAvatar,
    logout,
    updateUser,
    debugGetUsers
};